#!/usr/bin/env python3
import sys
import os
import logging
from logging.handlers import RotatingFileHandler
from threading import Lock

# Add the application root to the Python path
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from app import create_app
from controllers.message_controller import MessageController

# Configure logging
logger = logging.getLogger(__name__)
fetch_lock = Lock()

def setup_logging():
    os.makedirs('logs', exist_ok=True)
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
    
    file_handler = RotatingFileHandler(
        'logs/cron.log', 
        maxBytes=10000000,
        backupCount=5
    )
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    logger.setLevel(logging.INFO)

def fetch_messages():
    if not fetch_lock.acquire(blocking=False):
        logger.info("Previous fetch operation still in progress, skipping this run")
        return
        
    try:
        logger.info("Starting scheduled fetch operation")
        
        # First process chats
        chats_count = MessageController.process_chats()
        logger.info(f"Processed {chats_count} chats")
        
        # Then process messages
        new_messages_count = MessageController.process_new_messages()
        logger.info(f"Processed {new_messages_count} new messages")
        
    except Exception as e:
        logger.error(f"Error in scheduled fetch: {str(e)}", exc_info=True)
    finally:
        fetch_lock.release()
        logger.info("Completed scheduled fetch operation")

if __name__ == "__main__":
    setup_logging()
    app = create_app()
    with app.app_context():
        fetch_messages()